@@ -92,6 +92,15 @@ where
         semi!(self);
     }
 
+    #[emitter]
+    fn emit_import_source(&mut self, n: &ImportSource) -> Result {
+        match n {
+            ImportSource::Fn(n) => emit!(self, n),
+            ImportSource::Url(n) => emit!(self, n),
+            ImportSource::Str(n) => emit!(self, n),
+        }
+    }
+
     #[emitter]
     fn emit_import_rule(&mut self, n: &ImportRule) -> Result {
         punct!(self, "@");
@@ -357,8 +366,7 @@ where
 
     #[emitter]
     fn emit_text(&mut self, n: &Text) -> Result {
-        self.wr
-            .write_ident(Some(n.span), &n.value, self.ctx.escape_first_dash)?;
+        self.wr.write_raw(Some(n.span), &n.raw)?;
     }
 
     #[emitter]
@@ -485,10 +493,9 @@ where
         for TokenAndSpan { span, token } in &n.tokens {
             let span = *span;
             match token {
-                Token::AtKeyword(name) => {
+                Token::AtKeyword { raw, .. } => {
                     punct!(self, span, "@");
-                    punct!(self, span, "@");
-                    self.wr.write_ident(Some(span), &name, false)?;
+                    self.wr.write_raw(Some(n.span), &raw)?;
                 }
                 Token::LParen => {
                     punct!(self, span, "(");
@@ -508,8 +515,8 @@ where
                 Token::Num(n) => {
                     self.wr.write_raw(Some(span), &n.value.to_string())?;
                 }
-                Token::Ident(n) => {
-                    self.wr.write_ident(Some(span), &n, true)?;
+                Token::Ident { raw, .. } => {
+                    self.wr.write_raw(Some(n.span), &raw)?;
                 }
                 Token::Str { value } => {
                     punct!(self, "'");
@@ -712,10 +719,7 @@ where
             self.wr.write_punct(None, combinator.as_str())?;
         }
 
-        let ctx = Ctx {
-            escape_first_dash: true,
-            ..self.ctx
-        };
+        let ctx = Ctx { ..self.ctx };
         emit!(&mut *self.with_ctx(ctx), n.type_selector);
 
         self.emit_list(&n.subclass_selectors, ListFormat::NotDelimited)?;
@@ -754,20 +758,14 @@ where
     #[emitter]
     fn emit_id_selector(&mut self, n: &IdSelector) -> Result {
         punct!(self, "#");
-        let ctx = Ctx {
-            escape_first_dash: true,
-            ..self.ctx
-        };
+        let ctx = Ctx { ..self.ctx };
         emit!(&mut *self.with_ctx(ctx), n.text);
     }
 
     #[emitter]
     fn emit_class_selector(&mut self, n: &ClassSelector) -> Result {
         punct!(self, ".");
-        let ctx = Ctx {
-            escape_first_dash: true,
-            ..self.ctx
-        };
+        let ctx = Ctx { ..self.ctx };
         emit!(&mut *self.with_ctx(ctx), n.text);
     }
 
@@ -778,9 +776,7 @@ where
         emit!(self, n.name);
 
         if let Some(op) = n.op {
-            space!(self);
             self.wr.write_punct(None, op.as_str())?;
-            space!(self);
         }
 
         emit!(self, n.value);
